@@ -45,7 +45,7 @@
 
 /* BSDI has this functionality, but its not defined */
 #if !defined(RTLD_NEXT)
-#define RTLD_NEXT ((void *)-1L)
+#define RTLD_NEXT ((void *) -1L)
 #endif
 
 using namespace std;
@@ -123,17 +123,20 @@ typedef int (*fcn) (int *(main) (int, char **, char **), int argc, char ** argv,
 #endif
 typedef char * (*gfcn) (const char *);
 
-union Start {
+union Start
+{
 	void * d;
 	fcn f;
 } start; // symbol for libc pre-main
-union Sym {
+union Sym
+{
 	void * d;
 	gfcn f;
 } sym, ssym; // symbols for libc (secure) getenv
 
 typedef pid_t (*ffcn) (void);
-union Fork {
+union Fork
+{
 	void * d;
 	ffcn f;
 } ffork; // symbols for libc fork
@@ -743,4 +746,4 @@ extern "C" char * secure_getenv (const char * name) // throw ()
 	elektraUnlockMutex ();
 	return ret;
 }
-}
+} // namespace ckdb
