@@ -54,8 +54,7 @@ int elektraGlobMatch (Key * key, const Key * match, const char * globFlags)
 	return 0;
 }
 
-enum GlobDirection
-{
+enum GlobDirection {
 	GET,
 	SET,
 };
