@@ -51,6 +51,8 @@ typedef struct
 	int counter;
 } PluginConfig;
 
+
+// elektra wildcard syntax to fnmatch syntax
 static char * keyNameToMatchingString (const Key * key)
 {
 	uint8_t arrayCount = 0;
@@ -80,9 +82,17 @@ static char * keyNameToMatchingString (const Key * key)
 	return pattern;
 }
 
+
+// match pattern against keyname - ignore namespace
+// @retval true if keyname matches pattern
+// @retval false if keyname doesn't match pattern
 static int matchPatternToKey (const char * pattern, const Key * key)
 {
-	return !fnmatch (pattern, (strchr (keyName (key), '/') + 1), FNM_PATHNAME);
+	const char * ptr = strchr (keyName (key), '/');
+	if (ptr)
+		return !fnmatch (pattern, ++ptr, FNM_PATHNAME);
+	else
+		return FNM_NOMATCH;
 }
 
 static int isValidArrayKey (Key * key)
@@ -239,53 +249,53 @@ static void parseLocalConfig (Key * specKey, ConflictHandling * localCh, Directi
 	switch (dir)
 	{
 	case GET:
-		if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/get/member")) != NULL)
+		if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/get/member")) != NULL)
 		{
 			localCh->member = getConfOption (localConflictMeta);
 		}
-		else if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/get/invalid")) != NULL)
+		else if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/get/invalid")) != NULL)
 		{
 			localCh->invalid = getConfOption (localConflictMeta);
 		}
-		else if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/get/count")) != NULL)
+		else if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/get/count")) != NULL)
 		{
 			localCh->count = getConfOption (localConflictMeta);
 		}
-		else if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/get/collision")) != NULL)
+		else if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/get/collision")) != NULL)
 		{
 			localCh->conflict = getConfOption (localConflictMeta);
 		}
-		else if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/get/range")) != NULL)
+		else if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/get/range")) != NULL)
 		{
 			localCh->range = getConfOption (localConflictMeta);
 		}
-		else if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/get/missing")) != NULL)
+		else if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/get/missing")) != NULL)
 		{
 			localCh->missing = getConfOption (localConflictMeta);
 		}
 		break;
 	case SET:
-		if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/set/member")) != NULL)
+		if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/set/member")) != NULL)
 		{
 			localCh->member = getConfOption (localConflictMeta);
 		}
-		else if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/set/invalid")) != NULL)
+		else if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/set/invalid")) != NULL)
 		{
 			localCh->invalid = getConfOption (localConflictMeta);
 		}
-		else if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/set/count")) != NULL)
+		else if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/set/count")) != NULL)
 		{
 			localCh->count = getConfOption (localConflictMeta);
 		}
-		else if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/set/collision")) != NULL)
+		else if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/set/collision")) != NULL)
 		{
 			localCh->conflict = getConfOption (localConflictMeta);
 		}
-		else if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/set/range")) != NULL)
+		else if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/set/range")) != NULL)
 		{
 			localCh->range = getConfOption (localConflictMeta);
 		}
-		else if ((localConflictMeta = (Key *)keyGetMeta (specKey, "conflict/set/missing")) != NULL)
+		else if ((localConflictMeta = (Key *) keyGetMeta (specKey, "conflict/set/missing")) != NULL)
 		{
 			localCh->missing = getConfOption (localConflictMeta);
 		}
@@ -369,7 +379,7 @@ static int handleArrayConflict (Key * parentKey, Key * key, Key * conflictMeta,
 	}
 	if (problemKeys)
 	{
-		elektraFree ((void *)problemKeys);
+		elektraFree ((void *) problemKeys);
 	}
 	return ret;
 }
@@ -436,7 +446,7 @@ static int handleConflictConflict (Key * parentKey, Key * key, Key * conflictMet
 	}
 	if (problemKeys)
 	{
-		elektraFree ((void *)problemKeys);
+		elektraFree ((void *) problemKeys);
 	}
 	return ret;
 }
@@ -502,7 +512,7 @@ static int handleMissingConflict (Key * parentKey, Key * key, Key * conflictMeta
 	}
 	if (problemKeys)
 	{
-		elektraFree ((void *)problemKeys);
+		elektraFree ((void *) problemKeys);
 	}
 	return ret;
 }
@@ -551,7 +561,7 @@ static int handleErrors (Key * parentKey, KeySet * ks, Key * key, Key * specKey,
 	Key * meta;
 	while (keyNextMeta (parent) != NULL)
 	{
-		meta = (Key *)keyCurrentMeta (parent);
+		meta = (Key *) keyCurrentMeta (parent);
 		conflict = getConflict (meta);
 		if (conflict != NAC)
 		{
@@ -566,7 +576,7 @@ static int handleErrors (Key * parentKey, KeySet * ks, Key * key, Key * specKey,
 	keyRewindMeta (key);
 	while (keyNextMeta (key) != NULL)
 	{
-		meta = (Key *)keyCurrentMeta (key);
+		meta = (Key *) keyCurrentMeta (key);
 		conflict = getConflict (meta);
 		if (conflict != NAC)
 		{
@@ -959,7 +969,7 @@ Plugin * ELEKTRA_PLUGIN_EXPORT (spec)
 {
 	// clang-format off
 	return elektraPluginExport ("spec",
-		ELEKTRA_PLUGIN_CLOSE, &elektraSpecClose,
+			ELEKTRA_PLUGIN_CLOSE, &elektraSpecClose,
 			ELEKTRA_PLUGIN_GET,	&elektraSpecGet,
 			ELEKTRA_PLUGIN_SET,	&elektraSpecSet,
 			ELEKTRA_PLUGIN_END);
